from urllib.parse import urljoin

import requests
from requests_aws4auth import AWS4Auth


IVONA_REGION_ENDPOINTS = {
    'eu-west-1': 'https://tts.eu-west-1.ivonacloud.com',  # EU, Dublin
    'us-east-1': 'https://tts.us-east-1.ivonacloud.com',  # US East, N. Virginia
    'us-west-2': 'https://tts.us-west-2.ivonacloud.com',  # US West, Oregon
}


class IvonaAPI:
    """
    Base class that for accessing Ivona web API.
    Currently implements 'CreateSpeech' and 'ListVoices' endpoints, without
    support for lexicon actions.
    """
    ALLOWED_CODECS = ['ogg', 'mp3', 'mp4']

    @property
    def region(self):
        return self._region

    @region.setter
    def region(self, value):
        if value not in IVONA_REGION_ENDPOINTS.keys():
            raise ValueError("Incorrect region: {}".format(value))
        self._region = value

    @property
    def codec(self):
        return self._codec

    @codec.setter
    def codec(self, value):
        if value.lower() not in self.ALLOWED_CODECS:
            raise ValueError("Incorrect codec - only ogg, mp3 and mp4 allowed.")
        self._codec = value

    def __init__(self, access_key, secret_key, voice_name='Salli',
                 language='en-US', codec='mp3', region='eu-west-1'):
        self.region = region
        self._aws4auth = AWS4Auth(access_key, secret_key, region, 'tts')

        self.available_voices = self.get_available_voices()
        self.set_voice(voice_name, language)

        self.codec = codec.lower()

        # Below are listed additional parameters that have default values,
        # but are initialized here so that they can be changed on instance
        # basis if need be

        # ['x-slow', 'slow', 'medium', 'fast', 'x-fast', 'default']
        self.rate = 'default'
        # ['silent', 'x-soft', 'soft', 'medium', 'loud', 'x-loud', 'default']
        self.volume = 'default'
        # Integer in the range of 0-3000 (in milliseconds)
        self.sentence_break = 400
        # Integer in the range of 0-5000 (in milliseconds)
        self.paragraph_break = 650

    def _check_if_voice_exists(self, voice_name, language):
        if not any([v['Name'] == voice_name and v['Language'] == language
                    for v in self.available_voices]):
            return False
        else:
            return True

    def set_voice(self, voice_name, language):
        """Make sure that passed voice name and language pair exists"""
        if not self._check_if_voice_exists(voice_name, language):
            raise ValueError("Incorrect voice name-language pair.")
        self._voice_name = voice_name
        self._language = language

    def get_available_voices(self, filter_language=None):
        """
        Returns a list of available voices, via 'ListVoices' endpoint
            http://developer.ivona.com/en/speechcloud/actions.html#ListVoices
        """
        endpoint = urljoin(
            IVONA_REGION_ENDPOINTS[self.region], 'ListVoices',
        )
        if filter_language:
            if not any([v['Language'] == filter_language
                        for v in self.available_voices]):
                raise ValueError("Incorrect language.")

            data = {
                'Voice': {
                    'Language': filter_language,
                },
            }
            r = requests.post(endpoint, auth=self._aws4auth, json=data)
        else:
            r = requests.get(endpoint, auth=self._aws4auth)

        if 'x-amzn-ErrorType' in r.headers:
            raise IvonaAPIException(r.headers['x-amzn-ErrorType'])

        return r.json()['Voices']

    def text_to_speech(self, text, file, voice_name=None, language=None):
        """
        Saves given text synthesized audio file, via 'CreateSpeech' endpoint
            http://developer.ivona.com/en/speechcloud/actions.html#CreateSpeech
        """
        if voice_name or language:
            if not self._check_if_voice_exists(voice_name, language):
                raise ValueError("Incorrect voice name-language pair.")
        else:
            voice_name = self._voice_name
            language = self._language

        endpoint = urljoin(
            IVONA_REGION_ENDPOINTS[self.region], 'CreateSpeech',
        )

        data = {
            'Input': {
                'Data': str(text),
            },
            'OutputFormat': {
                'Codec': self.codec.upper(),
            },
            'Parameters': {
                'Rate': self.rate,
                'Volume': self.volume,
                'SentenceBreak': self.sentence_break,
                'ParagraphBreak': self.paragraph_break,
            },
            'Voice': {
                'Name': voice_name,
                'Language': language,
            },
        }

        r = requests.post(endpoint, auth=self._aws4auth, json=data)

        if 'x-amzn-ErrorType' in r.headers:
            raise IvonaAPIException(r.headers['x-amzn-ErrorType'])

        file.write(r.content)

        return True


class IvonaAPIException(Exception):
    """Base IvonaAPI exception"""
    pass



import os
import tempfile
import filecmp
from uuid import uuid4

import pytest
from flaky import flaky

from ivona_api.ivona_api import IvonaAPI, IvonaAPIException


# Module fixtures
@pytest.fixture(scope='module')
def auth_keys():
    """Get working auth keys from environment variables"""
    access_key = os.environ["IVONA_ACCESS_KEY"]
    secret_key = os.environ["IVONA_SECRET_KEY"]
    assert access_key and secret_key

    return access_key, secret_key


# Tests
def test_init(auth_keys):
    """Test initializing"""
    # No arguments
    with pytest.raises(TypeError):
        IvonaAPI()

    # Wrong auth keys
    with pytest.raises(IvonaAPIException):
        IvonaAPI(str(uuid4()), str(uuid4()))

    # With incorrect region
    with pytest.raises(ValueError):
        IvonaAPI(auth_keys[0], auth_keys[1], region=str(uuid4()))

    # With nonexistent voice
    with pytest.raises(ValueError):
        IvonaAPI(auth_keys[0], auth_keys[1], voice_name=str(uuid4()))

    # With not allowed codec
    with pytest.raises(ValueError):
        IvonaAPI(auth_keys[0], auth_keys[1], codec=str(uuid4()))


@flaky
def test_available_voices(auth_keys):
    """Test getting available voices"""
    ivona_api = IvonaAPI(auth_keys[0], auth_keys[1])

    voices = ivona_api.available_voices
    assert len(voices) > 1

    # Make sure that default voice is available
    assert any([v['Name'] == 'Salli' and v['Language'] == 'en-US'
                for v in voices])


@flaky
def test_available_voices_with_filter(auth_keys):
    """Test getting available voices with filtering"""
    ivona_api = IvonaAPI(auth_keys[0], auth_keys[1])

    with pytest.raises(ValueError):
        ivona_api.get_available_voices(str(uuid4()))

    voices = ivona_api.get_available_voices('en-US')
    assert len(voices) > 1

    # Make sure that default voice is available
    assert any([v['Name'] == 'Salli' and v['Language'] == 'en-US'
                for v in voices])


@flaky
@pytest.mark.parametrize('voice_name,voice_language,content,org_file', [
    ('Salli', 'en-US', 'Hello world', 'files/salli_hello_world.mp3'),
    ('Maja', 'pl-PL', 'Dzień dobry', 'files/maja_dzien_dobry.mp3'),
])
def test_text_to_speech(auth_keys, voice_name, voice_language, content,
                        org_file):
    """Test synthesizing text to audio files"""
    ivona_api = IvonaAPI(
        auth_keys[0], auth_keys[1],
        voice_name=voice_name, language=voice_language,
    )

    with tempfile.NamedTemporaryFile() as temp_file:
        ivona_api.text_to_speech(content, temp_file)

        assert filecmp.cmp(org_file, temp_file.name)


def test_text_to_speech_custom_voice(auth_keys):
    """Test setting custom voice"""
    ivona_api = IvonaAPI(auth_keys[0], auth_keys[1])

    with pytest.raises(ValueError):
        with tempfile.NamedTemporaryFile() as temp_file:
            ivona_api.text_to_speech(
                str(uuid4()), temp_file,
                voice_name=str(uuid4()),
            )



# -*- coding: utf-8 -*-
from __future__ import print_function
import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


class Tox(TestCommand):
    user_options = [('tox-args=', 'a', "Arguments to pass to tox")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.tox_args = None

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import tox
        import shlex
        args = self.tox_args
        if args:
            args = shlex.split(self.tox_args)
        errno = tox.cmdline(args=args)
        sys.exit(errno)

# Convert description from markdown to reStructuredText
try:
    import pypandoc
    description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError):
    description = ''


setup(
    name='ivona_api',
    url='https://github.com/Pythonity/python-ivona-api',
    download_url='https://github.com/Pythonity/python-ivona-api/releases/latest',
    bugtrack_url='https://github.com/Pythonity/python-ivona-api/issues',
    version='0.1.1',
    license='MIT License',
    author='Pythonity',
    author_email='pythonity@pythonity.com',
    maintainer='Paweł Adamczak',
    maintainer_email='pawel.adamczak@sidnet.info',
    description="Python library that helps you connect to Amazon's IVONA "
                "Speech Cloud from within your Python project.",
    long_description=description,
    packages=find_packages(),
    include_package_data=True,
    tests_require=['tox'],
    cmdclass={'test': Tox},
    install_requires=[
        'requests>=2.10.0',
        'requests-aws4auth>=0.9',
    ],
    extras_require={
        'testing': ['pytest', 'flaky'],
    },
    keywords='amazon ivona api text to speech',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
